#########################################################################
# winmmucl - Mark's MUd Client with a text widget to display output.
#
# Copyright (C) 1997-1998 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

source [file join $env(MMUCL_DIR) lib mmucl.tcl]
source [file join $env(MMUCL_DIR) lib mmucl_gui.tcl]
source [file join $env(MMUCL_DIR) lib form.tcl]
source [file join $env(MMUCL_DIR) lib balloon.tcl]

mmucl::gui_init

# Menubar
menu .menubar

# Main menu divisions.
.menubar add cascade -label File -menu .menubar.file -underline 0
.menubar add cascade -label Edit -menu .menubar.edit -underline 0
.menubar add cascade -label Options -menu .menubar.options -underline 0
.menubar add cascade -label Help -menu .menubar.help -underline 0

# File menu
menu .menubar.file

.menubar.file add command -label Connect -command mmucl::tk_connect
.menubar.file add separator
.menubar.file add command -label Load \
    -command {tk_openFile mmucl::load -title {Load a File}}
.menubar.file add command -label Send \
    -command {tk_openFile mmucl::MCtextin -title {Send a file to the mud}}
.menubar.file add command -label Save -command \
    {tk_saveFile mmucl::MCsave -title {Save Mmucl State}} 
.menubar.file add separator
.menubar.file add command -label Exit -command \
    {tk_dialogBox mmucl::MCexit yes -type yesno -message "Really quit?"}

# Edit menu
menu .menubar.edit

.menubar.edit add command -label Copy -command {tk_textCopy .mmucl_display}
.menubar.edit add command -label Paste -command {tk_textPaste .mmucl_input}
.menubar.edit add separator
.menubar.edit add command -label Aliases -command {mmucl::tk_edit alias}
.menubar.edit add command -label Actions -command {mmucl::tk_edit action}
.menubar.edit add command -label Chars -command {mmucl::tk_edit char}
.menubar.edit add command -label Keys -command {mmucl::tk_edit key}
.menubar.edit add command -label Subs -command {mmucl::tk_edit sub}

# Options menu
mmucl::option_menu .menubar.options

# Help menu
menu .menubar.help
 
.menubar.help add command -label About -command {mmucl::MChelp about}
.menubar.help add command -label Contents -command {mmucl::MChelp}

# Set .menubar as the toplevel menubar
. configure -menu .menubar

# toolbar

mmucl::toolbar .toolbar

# Create entry .mmucl_input for user input.
mmucl::user_entry .mmucl_input

# Bindings to scroll text.
bind .mmucl_input <Next> {.mmucl_display yview scroll 1 page}
bind .mmucl_input <Prior> {.mmucl_display yview scroll -1 page}
bind .mmucl_input <Key> {.mmucl_display see end}

text .mmucl_display
.mmucl_display configure -wrap word -yscrollcommand {.vscroll set}
scrollbar .vscroll -orient vertical -command {.mmucl_display yview}

grid .toolbar -row 0 -column 0  -sticky ew
grid .mmucl_display -row 1 -column 0 -sticky news
grid .vscroll -row 1 -column 1 -sticky ns
grid .mmucl_input -row 2 -column 0 -columnspan 1 -sticky news
grid columnconfigure . 0 -weight 1
grid rowconfigure . 1 -weight 1
wm title . Mmucl


# Callback to mmucl library that displays output from the mud.
# In this case the output is being displayed in a text widget
# so special processing is done. In particular ansi codes for
# color are mapped to text widget tags. Not completely correct,
# but close enough.

set mmucl::Mmucl(ansi_tags) {{} 37 40}

proc mmucl::display {str} {
    upvar #0 mmucl::Mmucl(ansi_tags) tags
    
    .mmucl_display configure -state normal
    set ypos [lindex [.mmucl_display yview] 1]

    regsub -all \r $str "" str
    foreach part [split $str \033] {
	if {[regexp {\[([0-9;]*)m(.*)} $part x codes text]} {
	    foreach code [split $codes \;] {
		if {$code == 0} {
		    set tags {{} {} {}}
		} elseif {$code < 8} {
		    set tags [lreplace $tags 0 0 $code]
		} elseif {$code < 40} {
		    set tags [lreplace $tags 1 1 $code]
		} else {
		    set tags [lreplace $tags 2 2 $code]
		}
	    }
	} else {
	    set text $part
	}
	.mmucl_display insert end $text $tags
    }

    if {$ypos == 1} {
    	.mmucl_display see end
    }
    if {[.mmucl_display index end] > 500} {
        .mmucl_display delete 1.0 50.0
    }
    .mmucl_display configure -state disabled
}

# bold font
.mmucl_display tag configure 1\
    -font [concat [lrange [.mmucl_display cget -font] 0 1] bold]
.mmucl_display tag configure 4  -underline 1

# Should blink
.mmucl_display tag configure 5 -underline 1

# Should reverse fg and bg
.mmucl_display tag configure 7 -background lightgrey -foreground black

.mmucl_display tag configure 30 -foreground darkgrey
.mmucl_display tag configure 31 -foreground red3
.mmucl_display tag configure 32 -foreground green3
.mmucl_display tag configure 33 -foreground yellow3
.mmucl_display tag configure 34 -foreground blue2
.mmucl_display tag configure 35 -foreground magenta3
.mmucl_display tag configure 36 -foreground cyan3
.mmucl_display tag configure 37 -foreground lightgrey
.mmucl_display tag configure 41 -background red3
.mmucl_display tag configure 42 -background green3
.mmucl_display tag configure 43 -background yellow3
.mmucl_display tag configure 44 -background blue2
.mmucl_display tag configure 45 -background magenta3
.mmucl_display tag configure 46 -background cyan3
.mmucl_display tag configure 47 -background lightgrey

# Make selection visible.
.mmucl_display tag raise sel

focus .mmucl_input

mmucl::init
